@@ -275,7 +275,7 @@ class POSIXTool(Bcfg2.Client.Tools.Tool):
         if path is None:
             path = entry.get("name")
         context = entry.get("secontext")
-        if context is None:
+        if not context:
             # no context listed
             return True
 
@@ -520,13 +520,19 @@ class POSIXTool(Bcfg2.Client.Tools.Tool):
                               "Current mtime is %s but should be %s" %
                               (path, mtime, entry.get('mtime')))
 
-        if HAS_SELINUX and entry.get("secontext"):
+        if HAS_SELINUX:
+            wanted_secontext = None
             if entry.get("secontext") == "__default__":
-                wanted_secontext = \
-                    selinux.matchpathcon(path, 0)[1].split(":")[2]
+                try:
+                    wanted_secontext = \
+                        selinux.matchpathcon(path, 0)[1].split(":")[2]
+                except OSError:
+                    errors.append("%s has no default SELinux context" %
+                                  entry.get("name"))
             else:
                 wanted_secontext = entry.get("secontext")
-            if attrib['current_secontext'] != wanted_secontext:
+            if (wanted_secontext and
+                attrib['current_secontext'] != wanted_secontext):
                 errors.append("SELinux context for path %s is incorrect. "
                               "Current context is %s but should be %s" %
                               (path, attrib['current_secontext'],
@@ -700,20 +706,14 @@ class POSIXTool(Bcfg2.Client.Tools.Tool):
                               (path, err))
             rv = False
 
-        # we need to make sure that we give +x to everyone who needs
-        # it.  E.g., if the file that's been distributed is 0600, we
-        # can't make the parent directories 0600 also; that'd be
-        # pretty useless.  They need to be 0700.
+        # set auto-created directories to mode 755, if you need
+        # something else, you should specify it in your config
         tmpentry = copy.deepcopy(entry)
-        newmode = int(entry.get('mode'), 8)
-        for i in range(0, 3):
-            if newmode & (6 * pow(8, i)):
-                newmode |= 1 * pow(8, i)
-        tmpentry.set('mode', oct_mode(newmode))
+        tmpentry.set('mode', '0755')
         for acl in tmpentry.findall('ACL'):
             acl.set('perms',
-                    oct_mode(self._norm_acl_perms(acl.get('perms')) | \
-                                 ACL_MAP['x']))
+                    oct_mode(self._norm_acl_perms(acl.get('perms')) |
+                             ACL_MAP['x']))
         for cpath in created:
             rv &= self._set_perms(tmpentry, path=cpath)
         return rv
