@@ -308,17 +308,39 @@ def list_split(c_string):
     """ split an option string on commas, optionally surrounded by
     whitespace, returning a list """
     if c_string:
-        return re.split("\s*,\s*", c_string)
+        return re.split(r'\s*,\s*', c_string)
     return []
 
 
 def colon_split(c_string):
     """ split an option string on colons, returning a list """
     if c_string:
-        return c_string.split(':')
+        return c_string.split(r':')
     return []
 
 
+def dict_split(c_string):
+    """ split an option string on commas, optionally surrounded by
+    whitespace and split the resulting items again on equals signs,
+    returning a dict """
+    result = dict()
+    if c_string:
+        items = re.split(r'\s*,\s*', c_string)
+        for item in items:
+            if r'=' in item:
+                key, value = item.split(r'=', 1)
+                try:
+                    result[key] = get_bool(value)
+                except ValueError:
+                    try:
+                        result[key] = get_int(value)
+                    except ValueError:
+                        result[key] = value
+            else:
+                result[item] = True
+    return result
+
+
 def get_bool(val):
     """ given a string value of a boolean configuration option, return
     an actual bool (True or False) """
@@ -334,12 +356,28 @@ def get_bool(val):
         raise ValueError("Not a boolean value", val)
 
 
+def get_int(val):
+    """ given a string value of an integer configuration option,
+    return an actual int """
+    return int(val)
+
+
+def get_timeout(val):
+    """ convert the timeout value into a float or None """
+    if val is None:
+        return val
+    timeout = float(val)  # pass ValueError up the stack
+    if timeout <= 0:
+        return None
+    return timeout
+
+
 def get_size(value):
     """ Given a number of bytes in a human-readable format (e.g.,
     '512m', '2g'), get the absolute number of bytes as an integer """
     if value == -1:
         return value
-    mat = re.match("(\d+)([KkMmGg])?", value)
+    mat = re.match(r'(\d+)([KkMmGg])?', value)
     if not mat:
         raise ValueError("Not a valid size", value)
     rvalue = int(mat.group(1))
@@ -385,7 +423,8 @@ CFILE = \
     Option('Specify configuration file',
            default=DEFAULT_CONFIG_LOCATION,
            cmd='-C',
-           odesc='<conffile>')
+           odesc='<conffile>',
+           env="BCFG2_CONFIG")
 LOCKFILE = \
     Option('Specify lockfile',
            default='/var/lock/bcfg2.run',
@@ -518,6 +557,11 @@ SERVER_FAM_IGNORE = \
                     'SCCS', '.svn', '4913', '.gitignore'],
            cf=('server', 'ignore_files'),
            cook=list_split)
+SERVER_FAM_BLOCK = \
+    Option('FAM blocks on startup until all events are processed',
+           default=False,
+           cook=get_bool,
+           cf=('server', 'fam_blocking'))
 SERVER_LISTEN_ALL = \
     Option('Listen on all interfaces',
            default=False,
@@ -557,7 +601,7 @@ SERVER_PASSWORD = \
 SERVER_PROTOCOL = \
     Option('Server Protocol',
            default='xmlrpc/ssl',
-           cf=('communication', 'procotol'))
+           cf=('communication', 'protocol'))
 SERVER_BACKEND = \
     Option('Server Backend',
            default='best',
@@ -582,6 +626,21 @@ SERVER_UMASK = \
            default='0077',
            odesc='<Server umask>',
            cf=('server', 'umask'))
+SERVER_AUTHENTICATION = \
+    Option('Default client authentication method',
+           default='cert+password',
+           odesc='{cert|bootstrap|cert+password}',
+           cf=('communication', 'authentication'))
+SERVER_CHILDREN = \
+    Option('Spawn this number of children for the multiprocessing core. '
+           'By default spawns children equivalent to the number of processors '
+           'in the machine.',
+           default=None,
+           cmd='--children',
+           odesc='<children>',
+           cf=('server', 'children'),
+           cook=get_int,
+           long_arg=True)
 
 # database options
 DB_ENGINE = \
@@ -615,6 +674,12 @@ DB_PORT = \
            cf=('database', 'port'),
            deprecated_cf=('statistics', 'database_port'))
 
+DB_OPTIONS = \
+    Option('Database options',
+           default=dict(),
+           cf=('database', 'options'),
+           cook=dict_split)
+
 # Django options
 WEB_CFILE = \
     Option('Web interface configuration file',
@@ -805,10 +870,20 @@ CLIENT_EXIT_ON_PROBE_FAILURE = \
            long_arg=True,
            cf=('client', 'exit_on_probe_failure'),
            cook=get_bool)
+CLIENT_PROBE_TIMEOUT = \
+    Option("Timeout when running client probes",
+           default=None,
+           cf=('client', 'probe_timeout'),
+           cook=get_timeout)
+CLIENT_COMMAND_TIMEOUT = \
+    Option("Timeout when client runs other external commands (not probes)",
+           default=None,
+           cf=('client', 'command_timeout'),
+           cook=get_timeout)
 
 # bcfg2-test and bcfg2-lint options
 TEST_NOSEOPTS = \
-    Option('Options to pass to nosetests',
+    Option('Options to pass to nosetests. Only honored with --children 0',
            default=[],
            cmd='--nose-options',
            odesc='<opts>',
@@ -823,6 +898,21 @@ TEST_IGNORE = \
            cf=('bcfg2_test', 'ignore_entries'),
            cook=list_split,
            long_arg=True)
+TEST_CHILDREN = \
+    Option('Spawn this number of children for bcfg2-test (python 2.6+)',
+           default=0,
+           cmd='--children',
+           odesc='<children>',
+           cf=('bcfg2_test', 'children'),
+           cook=get_int,
+           long_arg=True)
+TEST_XUNIT = \
+    Option('Output an XUnit result file with --children',
+           default=None,
+           cmd='--xunit',
+           odesc='<xunit file>',
+           cf=('bcfg2_test', 'xunit'),
+           long_arg=True)
 LINT_CONFIG = \
     Option('Specify bcfg2-lint configuration file',
            default='/etc/bcfg2-lint.conf',
@@ -1036,6 +1126,15 @@ VERBOSE = \
            cmd='-v',
            cook=get_bool,
            cf=('logging', 'verbose'))
+LOG_PERFORMANCE = \
+    Option("Periodically log performance statistics",
+           default=False,
+           cf=('logging', 'performance'))
+PERFLOG_INTERVAL = \
+    Option("Performance statistics logging interval in seconds",
+           default=300.0,
+           cook=get_timeout,
+           cf=('logging', 'performance_interval'))
 
 # Plugin-specific options
 CFG_VALIDATION = \
@@ -1110,6 +1209,7 @@ SERVER_COMMON_OPTIONS = dict(repo=SERVER_REPOSITORY,
                              password=SERVER_PASSWORD,
                              filemonitor=SERVER_FILEMONITOR,
                              ignore=SERVER_FAM_IGNORE,
+                             fam_blocking=SERVER_FAM_BLOCK,
                              location=SERVER_LOCATION,
                              key=SERVER_KEY,
                              cert=SERVER_CERT,
@@ -1117,7 +1217,11 @@ SERVER_COMMON_OPTIONS = dict(repo=SERVER_REPOSITORY,
                              protocol=SERVER_PROTOCOL,
                              web_configfile=WEB_CFILE,
                              backend=SERVER_BACKEND,
-                             vcs_root=SERVER_VCS_ROOT)
+                             vcs_root=SERVER_VCS_ROOT,
+                             authentication=SERVER_AUTHENTICATION,
+                             perflog=LOG_PERFORMANCE,
+                             perflog_interval=PERFLOG_INTERVAL,
+                             children=SERVER_CHILDREN)
 
 CRYPT_OPTIONS = dict(encrypt=ENCRYPT,
                      decrypt=DECRYPT,
@@ -1195,7 +1299,9 @@ CLIENT_COMMON_OPTIONS = \
          serverCN=CLIENT_SCNS,
          timeout=CLIENT_TIMEOUT,
          decision_list=CLIENT_DECISION_LIST,
-         probe_exit=CLIENT_EXIT_ON_PROBE_FAILURE)
+         probe_exit=CLIENT_EXIT_ON_PROBE_FAILURE,
+         probe_timeout=CLIENT_PROBE_TIMEOUT,
+         command_timeout=CLIENT_COMMAND_TIMEOUT)
 CLIENT_COMMON_OPTIONS.update(DRIVER_OPTIONS)
 CLIENT_COMMON_OPTIONS.update(CLI_COMMON_OPTIONS)
 
@@ -1207,6 +1313,7 @@ DATABASE_COMMON_OPTIONS = dict(web_configfile=WEB_CFILE,
                                db_password=DB_PASSWORD,
                                db_host=DB_HOST,
                                db_port=DB_PORT,
+                               db_options=DB_OPTIONS,
                                time_zone=DJANGO_TIME_ZONE,
                                django_debug=DJANGO_DEBUG,
                                web_prefix=DJANGO_WEB_PREFIX)
@@ -1214,6 +1321,17 @@ DATABASE_COMMON_OPTIONS = dict(web_configfile=WEB_CFILE,
 REPORTING_COMMON_OPTIONS = dict(reporting_file_limit=REPORTING_FILE_LIMIT,
                                 reporting_transport=REPORTING_TRANSPORT)
 
+TEST_COMMON_OPTIONS = dict(noseopts=TEST_NOSEOPTS,
+                           test_ignore=TEST_IGNORE,
+                           children=TEST_CHILDREN,
+                           xunit=TEST_XUNIT,
+                           validate=CFG_VALIDATION)
+
+INFO_COMMON_OPTIONS = dict(ppath=PARANOID_PATH,
+                           max_copies=PARANOID_MAX_COPIES)
+INFO_COMMON_OPTIONS.update(CLI_COMMON_OPTIONS)
+INFO_COMMON_OPTIONS.update(SERVER_COMMON_OPTIONS)
+
 
 class OptionParser(OptionSet):
     """
